import React, {
  createContext,
  useCallback,
  useContext,
  useEffect,
  useRef,
  useState
} from "react";
import {
  AppBar,
  Avatar,
  Button,
  Chip,
  IconButton,
  ListItem,
  ListItemIcon,
  ListItemText,
  makeStyles,
  Toolbar,
  Typography,
  Menu,
  MenuItem,
  Tooltip,
  Badge,
  Popover,
  Grid,
  List,
  Box,
  Divider,
  useTheme
} from "@material-ui/core";
import { withStyles } from "@material-ui/core/styles";
import {
  LockOpen,
  Home,
  Assignment,
  Storage,
  TrendingUp,
  SettingsInputComponent
} from "@material-ui/icons";
import HelpIcon from "@material-ui/icons/Help";
import { DeploymentStatusIcon } from "../../deployments/DeploymentIcons";
import DeleteIcon from "@material-ui/icons/Delete";
import NotificationsIcon from "@material-ui/icons/Notifications";
import { MenuDrawer } from "../Menu/MenuDrawer";
import { Link } from "react-router-dom";
import { userContext } from "../../../api/SwaggerApi";
import { notificationContext } from "../../common/notification/Notifications";
import { useWindowDimensions } from "../../common/Helper";
import { CCCEControlSymbol } from "../../../icons/CCCEControlSymbol";
import { applicationSubTitle } from "../../common/Helper";
import { LoginDialog } from "../../auth/Login";
import { useRedirect } from "../../../hooks/Redirect";

const useStyles = makeStyles((theme) => ({
  root: {
    flexGrow: 1,
    paddingBottom: theme.spacing(2)
  },
  appBar: {
    zIndex: 1250 // fixme: a hack to make the appBar stay on top of the drawer, should be between 1201 - 1299
  },
  menuButton: {
    marginRight: theme.spacing(2)
  },
  helpDivider: {
    marginTop: theme.spacing(5)
  },
  title: {
    flexGrow: 1
  },
  menuIcon: {
    marginLeft: theme.spacing(1)
  },
  typography: {
    padding: theme.spacing(2)
  },
  notificationButton: {
    marginRight: theme.spacing(2)
  },
  content: {
    marginTop: theme.spacing(1)
  },
  notificationList: {
    overflow: "auto",
    maxHeight: 360
  },
  shiftContentLeft: {
    marginLeft: theme.drawer.widthClose
  },
  shiftContentRight: {
    marginLeft: theme.drawer.widthOpen
  }
}));

const StyledMenu = withStyles({
  paper: {
    border: "1px solid #d3d4d5"
  }
})((props) => (
  <>
    <div style={{ ...props.theme.mixins.toolbar }} />
    <Menu
      elevation={0}
      getContentAnchorEl={null}
      anchorOrigin={{
        vertical: "bottom",
        horizontal: "center"
      }}
      transformOrigin={{
        vertical: "top",
        horizontal: "center"
      }}
      {...props}
    />
  </>
));

const StyledMenuItem = withStyles((theme) => ({
  root: {
    "&:focus": {
      backgroundColor: theme.palette.common.gray,
      "& .MuiListItemIcon-root, & .MuiListItemText-primary": {
        color: theme.palette.common.black
      }
    }
  }
}))(MenuItem);

function InfoMenuItem({ children }) {
  return (
    <MenuItem
      disabled
      style={{ opacity: 1.0 }}
    >
      {children}
    </MenuItem>
  );
}

function RBACRoleInfo({ role }) {
  return (
    <InfoMenuItem key={role}>
      <Chip
        label={role}
        color="secondary"
      />
    </InfoMenuItem>
  );
}

export function LoginControls() {
  const [loginFormOpen, setLoginFormOpen] = useState(false);
  const { user, login, loginError, resetLoginError } = useContext(userContext);
  const userRef = useRef(user);
  const redirect = useRedirect();

  const handleClose = useCallback(() => {
    setLoginFormOpen(false);
  }, []);

  // detect logout and redirect to login
  useEffect(() => {
    if (userRef.current && !user) {
      redirect("/login", {}, true);
    }
    userRef.current = user;
  }, [redirect, user]);

  useEffect(() => {
    if (user) {
      handleClose();
    }
  }, [handleClose, user]);

  return user ? (
    <>
      <NotificationMenu />
      <ProfileMenu />
    </>
  ) : (
    <>
      <Button
        variant="outlined"
        color="inherit"
        onClick={() => {
          resetLoginError();
          setLoginFormOpen(true);
        }}
      >
        Login
      </Button>
      <LoginDialog
        login={login}
        open={loginFormOpen}
        handleClose={handleClose}
        error={loginError}
        resetError={resetLoginError}
      />
    </>
  );
}

export function ButtonAppBar({ homeUrl, homeClick, title, button }) {
  const classes = useStyles();

  return (
    <div className={classes.root}>
      <AppBar
        position="fixed"
        className={classes.appBar}
      >
        <Toolbar>
          <IconButton
            edge="start"
            component={Link}
            to={homeUrl}
            onClick={homeClick}
            className={classes.menuButton}
            color="inherit"
            aria-label="menu"
          >
            <Home />
          </IconButton>
          <Typography
            variant="h1"
            className={classes.title}
            noWrap
          >
            {title}
          </Typography>
          {button}
        </Toolbar>
      </AppBar>
    </div>
  );
}

const defaultTitle = "CE deploy & monitor";
export const GlobalAppBarContext = createContext({
  title: defaultTitle,
  setButton: () => {
    console.info("default GlobalAppBarContext.setButton does nothing.");
  },
  setTitle: () => {
    console.info("default GlobalAppBarContext.setTitle does nothing.");
  }
});

function MenuListItem({ url, icon, text, tooltip }) {
  const classes = useStyles();
  const currentUrl = `${window.location}`;
  return (
    <>
      {url.includes("help") ? (
        <Divider className={classes.helpDivider} />
      ) : (
        <></>
      )}
      <Tooltip
        title={tooltip ? text : ""}
        placement="right"
        arrow
      >
        <ListItem
          button
          component={Link}
          to={url}
          selected={currentUrl.split("?")[0].endsWith(url)}
        >
          <ListItemIcon className={classes.menuIcon}>{icon}</ListItemIcon>
          <ListItemText
            primary={text}
            primaryTypographyProps={{ variant: "h5" }}
          />
        </ListItem>
      </Tooltip>
    </>
  );
}

function MenuListItems({ menuItems, drawerOpen }) {
  return (
    <>
      {menuItems.map(({ text, url, icon }) => (
        <MenuListItem
          key={text}
          tooltip={!drawerOpen}
          url={url}
          icon={icon}
          text={text}
        />
      ))}
    </>
  );
}

export function UserInfo({ user }) {
  return (
    <ListItem>
      <ListItemIcon>
        <Avatar
          src={user.avatar}
          variant="rounded"
        />
      </ListItemIcon>
      <ListItemText
        primary={user.fullName}
        secondary={user.email}
      />
    </ListItem>
  );
}

export function ProfileMenu() {
  const theme = useTheme();
  const { user, userRoles, logout } = useContext(userContext);
  const [anchorEl, setAnchorEl] = useState(null);

  const handleClick = (event) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  return (
    <>
      <Avatar
        src={user.avatar}
        variant="circular"
        onClick={handleClick}
        aria-controls="profile-menu"
        aria-haspopup="true"
      />
      <StyledMenu
        id="profile-menu"
        anchorEl={anchorEl}
        keepMounted
        open={Boolean(anchorEl)}
        onClose={handleClose}
        theme={theme}
      >
        {userRoles
          .filter((role) => role.includes("DeploymentTool"))
          .map((role) => (
            <RBACRoleInfo
              role={role}
              key={role}
            />
          ))}
        <StyledMenuItem
          onClick={() => {
            handleClose();
            logout();
          }}
        >
          <ListItemIcon>
            <LockOpen fontSize="small" />
          </ListItemIcon>
          <ListItemText
            primary="Logout"
            primaryTypographyProps={{ variant: "h5" }}
          />
        </StyledMenuItem>
      </StyledMenu>
    </>
  );
}

const StyledListItem = withStyles(() => ({
  root: {
    margin: 0,
    paddingLeft: 10
  }
}))(ListItem);

const StyledListItemIcon = withStyles(() => ({
  root: {
    minWidth: 0,
    marginRight: 12
  }
}))(ListItemIcon);

export function NotificationMenu() {
  const [anchorEl, setAnchorEl] = useState(null);
  const { user } = useContext(userContext);
  const classes = useStyles();
  const { notifications, clearNotifications } = useContext(notificationContext);

  const handleClick = (event) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  function haveNotification() {
    return (
      <>
        <Grid
          item
          xs={12}
        >
          <List className={classes.notificationList}>
            {notifications.map((notification) => {
              return (
                <>
                  {notification.link ? (
                    <StyledListItem
                      button
                      component={Link}
                      to={notification.link}
                      onClick={() => {
                        handleClose();
                      }}
                    >
                      <StyledListItemIcon className={classes.notificationItem}>
                        <DeploymentStatusIcon status={notification.status} />
                      </StyledListItemIcon>
                      <Typography className={classes.notificationItem}>
                        {notification.message}
                      </Typography>
                    </StyledListItem>
                  ) : (
                    <StyledListItem key={notification.message}>
                      <StyledListItemIcon className={classes.notificationItem}>
                        <DeploymentStatusIcon status={notification.status} />
                      </StyledListItemIcon>
                      <Typography className={classes.notificationItem}>
                        {notification.message}
                      </Typography>
                    </StyledListItem>
                  )}
                </>
              );
            })}
          </List>
        </Grid>
        <Grid
          item
          xs={1}
        >
          <Box
            display="flex"
            justifyContent="flex-end"
          >
            <IconButton
              color="inherit"
              onClick={() => {
                clearNotifications();
                handleClose();
              }}
            >
              <Tooltip
                title="Clear all notifications"
                arrow
              >
                <DeleteIcon fontSize="small" />
              </Tooltip>
            </IconButton>
          </Box>
        </Grid>
      </>
    );
  }

  function noNotification() {
    return (
      <Typography
        inset="true"
        style={{ margin: "4px" }}
      >
        You have no notifications!
      </Typography>
    );
  }

  return (
    <>
      {user ? (
        <>
          <IconButton
            color="inherit"
            onClick={handleClick}
            className={classes.notificationButton}
          >
            <Badge
              badgeContent={notifications.length}
              color="secondary"
            >
              <NotificationsIcon />
            </Badge>
          </IconButton>
          <Popover
            open={Boolean(anchorEl)}
            anchorEl={anchorEl}
            onClose={handleClose}
            anchorOrigin={{
              vertical: "bottom",
              horizontal: "center"
            }}
            transformOrigin={{
              vertical: "top",
              horizontal: "center"
            }}
          >
            <Grid
              container
              spacing={0}
              justify="flex-end"
            >
              {notifications.length > 0 ? haveNotification() : noNotification()}
            </Grid>
          </Popover>
        </>
      ) : (
        <></>
      )}
    </>
  );
}

export const DRAWER_OPEN = "DRAWER_OPEN";

const defaultButton = <LoginControls />;

export function GlobalAppBar({ children }) {
  const theme = useTheme();
  const classes = useStyles();
  const [button, setButton] = useState(defaultButton);
  const [title, setTitle] = useState(defaultTitle);
  const [value] = useState({ setButton, setTitle });
  const { width } = useWindowDimensions();
  const [drawerOpen, setDrawerOpen] = useState(isDrawerOpen() ?? width > 1730);

  useEffect(() => {
    if (isDrawerOpen()) {
      setDrawerOpen(width > 1730);
    }
  }, [width, setDrawerOpen]);

  const toggleDrawer = () => {
    storeDrawerOpen(!drawerOpen);
    setDrawerOpen(!drawerOpen);
  };

  function isDrawerOpen() {
    return JSON.parse(localStorage.getItem(DRAWER_OPEN));
  }

  function storeDrawerOpen(open) {
    localStorage.setItem(DRAWER_OPEN, JSON.stringify(open));
  }

  const makeLink = (text, url, icon) => ({ text, url, icon });
  const menuItemsAll = [
    makeLink("Records", "/records", <SettingsInputComponent />),
    makeLink("IOCs", "/iocs", <CCCEControlSymbol />),
    makeLink("IOC hosts", "/hosts", <Storage />),
    makeLink("Log", "/jobs", <Assignment />),
    makeLink("Statistics", "/statistics", <TrendingUp />),
    makeLink("Help", "/help", <HelpIcon />)
  ];

  return (
    <>
      <ButtonAppBar
        homeUrl="/home"
        title={title}
        button={button}
      />
      <div>
        <MenuDrawer
          open={drawerOpen}
          toggleDrawer={toggleDrawer}
        >
          <MenuListItems
            menuItems={menuItemsAll}
            drawerOpen={drawerOpen}
          />
        </MenuDrawer>
      </div>
      <div>
        <GlobalAppBarContext.Provider value={value}>
          <div style={{ ...theme.mixins.toolbar }} />
          <div className={classes.content}>
            <div
              className={
                drawerOpen
                  ? classes.shiftContentRight
                  : classes.shiftContentLeft
              }
            >
              {children}
            </div>
          </div>
        </GlobalAppBarContext.Provider>
      </div>
    </>
  );
}

export function useGlobalAppBar(initTitle, initButton = defaultButton) {
  const [button, setButton] = useState(initButton);
  const [title, setTitle] = useState(initTitle);
  const appBar = useContext(GlobalAppBarContext);
  useEffect(() => {
    appBar.setTitle(
      "CE deploy & monitor " + applicationSubTitle() + " / " + title
    );
    appBar.setButton(button);
  }, [appBar, title, button]);
  return { setButton, setTitle };
}
