import React, { useState, useEffect, useCallback } from "react";
import {
  Paper,
  Link,
  Container,
  IconButton,
  Card,
  CardContent,
  Typography
} from "@material-ui/core";
import { makeStyles } from "@material-ui/core/styles";
import ArrowBackIcon from "@material-ui/icons/ArrowBack";
import { useHostIOCList } from "../../api/SwaggerApi";
import { HostBadge } from "../../components/host/HostBadge";
import { SimpleAccordion } from "../../components/common/Accordion/SimpleAccordion";
import { IOCAsyncList } from "../../components/IOC/IOCAsyncList";
import { KeyValueTable } from "../../components/common/KeyValueTable/KeyValueTable";
import { LokiPanel } from "../../components/common/Loki/LokiPanel";
import { useNavigate } from "react-router-dom";
import { initRequestParams } from "../../components/common/Helper";
import AccessControl from "../../components/auth/AccessControl";
import { useGlobalAppBar } from "../../components/navigation/GlobalAppBar/GlobalAppBar";
import { useHost } from "../../api/SwaggerApi";
import AlertMessages from "../../components/IOC/AlertMessages";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";

const useStyles = makeStyles((theme) => ({
  secondItem: {
    marginTop: theme.spacing(2)
  },
  hostHeader: {
    marginTop: theme.spacing(2)
  }
}));

export function HostDetailsView({ id }) {
  const [host] = useHost(id);
  const [iocs, getIocs /* reset*/, , loading] = useHostIOCList();
  const [deployedIocs, setDeployedIocs] = useState([]);

  const [state, setState] = useUrlState(
    {
      iocs_rows: "20",
      iocs_page: "0",
      iocs_open: "true",
      syslog_open: "true",
      details_open: "false"
    },
    { navigateMode: "replace" }
  );

  const navigate = useNavigate();
  const classes = useStyles();

  const lazyParams = useCallback(() => {
    return {
      rows: deserialize(state.iocs_rows),
      page: deserialize(state.iocs_page)
    };
  }, [state]);

  const setLazyParams = useCallback(
    (params) => {
      setState({
        iocs_rows: serialize(params.rows),
        iocs_page: serialize(params.page)
      });
    },
    [setState]
  );

  const [columnSort, setColumnSort] = useState({
    sortField: null,
    sortOrder: null
  });

  const rowsPerPage = [20, 50];

  useEffect(() => {
    let requestParams = initRequestParams(lazyParams(), null, columnSort);

    requestParams.host_csentry_id = id;

    getIocs(requestParams);
  }, [getIocs, lazyParams, columnSort, id]);

  useEffect(() => {
    setDeployedIocs(iocs?.deployedList ?? []);
  }, [iocs, setDeployedIocs]);

  function listToString(list) {
    if (list) {
      if (Array.isArray(list)) {
        return list.join(", ");
      }
      return list;
    }

    return "";
  }

  let renderHost = {
    "registered by": host?.csEntryHost?.user,
    "device type": host?.csEntryHost?.device_type,
    description: host?.csEntryHost?.description,
    scope: host?.csEntryHost?.scope,
    "vm owner": listToString(host?.csEntryHost?.ansible_vars?.vm_owner),
    "ansible groups": listToString(host?.csEntryHost?.ansible_groups)
  };

  const handleClick = (event) => {
    navigate(-1);
  };

  const { setTitle } = useGlobalAppBar("Host Details");

  useEffect(() => {
    host &&
      host.csEntryHost &&
      setTitle("Host Details: " + host.csEntryHost.name);
  }, [host, setTitle]);

  return (
    <Paper>
      <IconButton
        color="inherit"
        onClick={handleClick}
      >
        <ArrowBackIcon />
      </IconButton>

      {host && <AlertMessages alerts={host.alerts} />}
      <Card
        variant="outlined"
        className={classes.hostHeader}
      >
        <CardContent>{host && <HostBadge host={host} />}</CardContent>
      </Card>
      <SimpleAccordion
        summary="Deployed IOCs"
        expanded={deserialize(state.iocs_open)}
        onChange={(event, expanded) =>
          setState({ iocs_open: serialize(expanded) })
        }
      >
        <IOCAsyncList
          iocs={deployedIocs}
          setIocs={setDeployedIocs}
          loading={loading}
          rowType="host"
          totalCount={iocs.totalCount}
          lazyParams={lazyParams()}
          setLazyParams={setLazyParams}
          columnSort={columnSort}
          setColumnSort={setColumnSort}
          rowsPerPage={rowsPerPage}
        />
      </SimpleAccordion>
      <SimpleAccordion
        summary="Host details"
        expanded={deserialize(state.details_open)}
        onChange={(event, expanded) =>
          setState({ details_open: serialize(expanded) })
        }
      >
        <AccessControl
          allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
          renderNoAccess={() => <></>}
        >
          {host && <KeyValueTable obj={renderHost} />}
        </AccessControl>
      </SimpleAccordion>
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        <SimpleAccordion
          summary="Syslog info"
          expanded={deserialize(state.syslog_open)}
          onChange={(event, expanded) =>
            setState({ syslog_open: serialize(expanded) })
          }
        >
          {host && (
            <Container>
              <LokiPanel
                host={host}
                isSyslog
                isDeployed
              />
            </Container>
          )}
        </SimpleAccordion>
      </AccessControl>
      <Card variant="outlined">
        <CardContent>
          <Container>
            <Typography variant="h6">Open in CSEntry</Typography>
            <Link
              href={`https://csentry.esss.lu.se/network/hosts/view/${host?.csEntryHost.name}`}
              target="_blank"
              rel="noreferrer"
            >{`https://csentry.esss.lu.se/network/hosts/view/${host?.csEntryHost?.name}`}</Link>

            <Typography
              variant="h6"
              className={classes.secondItem}
            >
              Open in Grafana
            </Typography>
            <Link
              href={`https://grafana.tn.esss.lu.se/d/5zJT23xWz/node-exporter-full?orgId=1&var-node=${host?.csEntryHost.fqdn}`}
              target="_blank"
              rel="noreferrer"
            >{`https://grafana.tn.esss.lu.se/d/5zJT23xWz/node-exporter-full?orgId=1&var-node=${host?.csEntryHost.fqdn}`}</Link>
          </Container>
        </CardContent>
      </Card>
    </Paper>
  );
}
