import React, { useContext, useState, useCallback } from "react";
import { makeStyles } from "@material-ui/core/styles";
import {
  Paper,
  Grid,
  FormControlLabel,
  Switch,
  Typography,
  Container,
  TextField
} from "@material-ui/core";
import Tabs from "@material-ui/core/Tabs";
import Tab from "@material-ui/core/Tab";
import { useOperationsSearch, userContext } from "../../api/SwaggerApi";
import { initRequestParams } from "../../components/common/Helper";
import { useEffect } from "react";
import { SearchBar } from "../../components/common/SearchBar/SearchBar";
import AccessControl from "../../components/auth/AccessControl";
import { Autocomplete } from "@material-ui/lab";
import { JobAsyncList } from "../../components/Job/JobAsyncList";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";

const useStyles = makeStyles((theme) => ({
  root: {
    width: "100%"
  },
  paper: {
    marginBottom: theme.spacing(2)
  },
  formControl: {
    marginLeft: "5px",
    marginRight: "5px"
  }
}));

export function JobListView() {
  const classes = useStyles();
  const [operations, getOperations /* reset*/, , loading] =
    useOperationsSearch();
  const [state, setState] = useUrlState(
    {
      tab: "0",
      own: "false",
      rows: "20",
      page: "0",
      query: "",
      job_type: null
    },
    { navigateMode: "replace" }
  );

  const { user } = useContext(userContext);
  const [deploymentStatus, setDeploymentStatus] = useState(null);
  const [jobList, setJobList] = useState([]);

  const handleTabChange = (event, tab) => {
    setState((s) =>
      serialize(s.tab) === serialize(tab)
        ? { tab: serialize(tab) }
        : { tab: serialize(tab), page: "0" }
    );

    changeTab(tab);
  };

  const changeTab = (tab) => {
    if (tab === 0) {
      setDeploymentStatus(null);
    } else if (tab === 1) {
      setDeploymentStatus("ONGOING");
    } else if (tab === 2) {
      setDeploymentStatus("FINISHED");
    }
  };

  useEffect(() => {
    state.tab && changeTab(deserialize(state.tab));
  }, [state]);

  const handleChangeOwn = (event) => {
    const own = event.target.checked;
    setState({ own: serialize(own), page: "0" });
  };

  const lazyParams = useCallback(() => {
    return {
      rows: deserialize(state.rows),
      page: deserialize(state.page)
    };
  }, [state]);

  const setLazyParams = useCallback(
    (params) => {
      setState({ rows: serialize(params.rows), page: serialize(params.page) });
    },
    [setState]
  );

  const rowsPerPage = [20, 50];

  useEffect(() => {
    let requestParams = initRequestParams(
      lazyParams(),
      deserialize(state.query)
    );

    if (deserialize(state.own)) {
      requestParams.user = user?.loginName;
    }

    if (deploymentStatus) {
      requestParams.status = deploymentStatus;
    }

    if (state.job_type) {
      requestParams.type = deserialize(state.job_type);
    }

    getOperations(requestParams);
  }, [getOperations, lazyParams, state, user, deploymentStatus]);

  useEffect(() => {
    setJobList(operations.operationsList);
  }, [operations, setJobList]);

  const setSearch = useCallback(
    (query) => {
      setState({ query: serialize(query) });
    },
    [setState]
  );

  const content = (
    <SearchBar
      search={setSearch}
      query={deserialize(state.query)}
      loading={loading}
      placeholder="Search in IOC name, user or git reference"
    >
      <JobAsyncList
        jobs={jobList}
        setJobs={setJobList}
        loading={loading}
        totalCount={operations.totalCount}
        lazyParams={lazyParams()}
        setLazyParams={setLazyParams}
        rowsPerPage={rowsPerPage}
      />
    </SearchBar>
  );

  const jobTypeOptions = [
    { label: "Show all" },
    { label: "Only deployments", filter: "DEPLOYMENT" },
    { label: "Only commands", filter: "COMMAND" }
  ];

  return (
    <Paper className={classes.paper}>
      <Grid
        container
        spacing={1}
      >
        <Grid
          container
          spacing={1}
          component={Container}
          justify="space-between"
          alignItems="center"
          style={{ display: "flex" }}
        >
          <Grid item>
            <Tabs
              value={deserialize(state.tab)}
              onChange={handleTabChange}
              indicatorColor="primary"
              textColor="primary"
            >
              <Tab label={<Typography variant="h5">All</Typography>} />
              <Tab label={<Typography variant="h5">Only Ongoing</Typography>} />
              <Tab
                label={<Typography variant="h5">Only Completed</Typography>}
              />
            </Tabs>
          </Grid>
          <Grid
            item
            xs={3}
          >
            <Autocomplete
              id="combo-box-demo"
              options={jobTypeOptions}
              defaultValue={
                state.job_type
                  ? jobTypeOptions.find((o) => o.filter === state.job_type) || {
                      label: "Show all"
                    }
                  : { label: "Show all" }
              }
              getOptionLabel={(option) => option.label}
              getOptionSelected={(option, value) =>
                option.label === value.label
              }
              style={{ width: "100%" }}
              renderInput={(params) => (
                <TextField
                  {...params}
                  label="Job type"
                  variant="outlined"
                />
              )}
              onChange={(event, value, reason) =>
                setState({ job_type: value?.filter })
              }
              autoSelect
            />
          </Grid>
          <Grid item>
            <AccessControl
              allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
              renderNoAccess={() => <></>}
            >
              <FormControlLabel
                className={classes.formControl}
                control={
                  <Switch
                    checked={deserialize(state.own)}
                    onChange={handleChangeOwn}
                  />
                }
                label={<Typography variant="h5">Only my jobs</Typography>}
              />
            </AccessControl>
          </Grid>
        </Grid>
        <Grid
          item
          xs={12}
          md={12}
        >
          {content}
        </Grid>
      </Grid>
    </Paper>
  );
}
